const ZAMStmt ZAMCompiler::AssignVC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_RECORD )
		z = GenInst(OP_ASSIGN_VC_R, n, c);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_ASSIGN_VC_P, n, c);
	else if ( tag == TYPE_OPAQUE )
		z = GenInst(OP_ASSIGN_VC_O, n, c);
	else if ( tag == TYPE_VECTOR )
		z = GenInst(OP_ASSIGN_VC_V, n, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_ASSIGN_VC_U, n, c);
	else if ( tag == TYPE_TYPE )
		z = GenInst(OP_ASSIGN_VC_t, n, c);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_ASSIGN_VC_D, n, c);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_ASSIGN_VC_T, n, c);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_ASSIGN_VC_N, n, c);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_ASSIGN_VC_A, n, c);
	else if ( tag == TYPE_ANY )
		z = GenInst(OP_ASSIGN_VC_a, n, c);
	else if ( tag == TYPE_LIST )
		z = GenInst(OP_ASSIGN_VC_L, n, c);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_ASSIGN_VC_F, n, c);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_ASSIGN_VC_S, n, c);
	else if ( tag == TYPE_FILE )
		z = GenInst(OP_ASSIGN_VC_f, n, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_ASSIGN_VC_I, n, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AssignVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n1->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_RECORD )
		z = GenInst(OP_ASSIGN_VV_R, n1, n2);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_ASSIGN_VV_P, n1, n2);
	else if ( tag == TYPE_OPAQUE )
		z = GenInst(OP_ASSIGN_VV_O, n1, n2);
	else if ( tag == TYPE_VECTOR )
		z = GenInst(OP_ASSIGN_VV_V, n1, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_ASSIGN_VV_U, n1, n2);
	else if ( tag == TYPE_TYPE )
		z = GenInst(OP_ASSIGN_VV_t, n1, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_ASSIGN_VV_D, n1, n2);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_ASSIGN_VV_T, n1, n2);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_ASSIGN_VV_N, n1, n2);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_ASSIGN_VV_A, n1, n2);
	else if ( tag == TYPE_ANY )
		z = GenInst(OP_ASSIGN_VV_a, n1, n2);
	else if ( tag == TYPE_LIST )
		z = GenInst(OP_ASSIGN_VV_L, n1, n2);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_ASSIGN_VV_F, n1, n2);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_ASSIGN_VV_S, n1, n2);
	else if ( tag == TYPE_FILE )
		z = GenInst(OP_ASSIGN_VV_f, n1, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_ASSIGN_VV_I, n1, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Field_LHS_AssignFC(const FieldLHSAssignExpr* flhs, const ConstExpr* c)
	{
	ZInstI z;
	auto n = flhs->GetOp1()->AsNameExpr();
	auto t = flhs->GetType();
	int field = flhs->Field();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_RECORD )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FC_R, n, c, field);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FC_P, n, c, field);
	else if ( tag == TYPE_OPAQUE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FC_O, n, c, field);
	else if ( tag == TYPE_VECTOR )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FC_V, n, c, field);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FC_U, n, c, field);
	else if ( tag == TYPE_TYPE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FC_t, n, c, field);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FC_D, n, c, field);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FC_T, n, c, field);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FC_N, n, c, field);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FC_A, n, c, field);
	else if ( tag == TYPE_ANY )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FC_a, n, c, field);
	else if ( tag == TYPE_LIST )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FC_L, n, c, field);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FC_F, n, c, field);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FC_S, n, c, field);
	else if ( tag == TYPE_FILE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FC_f, n, c, field);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FC_I, n, c, field);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Field_LHS_AssignFV(const FieldLHSAssignExpr* flhs, const NameExpr* n2)
	{
	ZInstI z;
	auto n1 = flhs->GetOp1()->AsNameExpr();
	auto t = flhs->GetType();
	int field = flhs->Field();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_RECORD )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FV_R, n1, n2, field);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FV_P, n1, n2, field);
	else if ( tag == TYPE_OPAQUE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FV_O, n1, n2, field);
	else if ( tag == TYPE_VECTOR )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FV_V, n1, n2, field);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FV_U, n1, n2, field);
	else if ( tag == TYPE_TYPE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FV_t, n1, n2, field);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FV_D, n1, n2, field);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FV_T, n1, n2, field);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FV_N, n1, n2, field);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FV_A, n1, n2, field);
	else if ( tag == TYPE_ANY )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FV_a, n1, n2, field);
	else if ( tag == TYPE_LIST )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FV_L, n1, n2, field);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FV_F, n1, n2, field);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FV_S, n1, n2, field);
	else if ( tag == TYPE_FILE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FV_f, n1, n2, field);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FV_I, n1, n2, field);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Field_LHS_AssignFVi(const FieldLHSAssignExpr* flhs, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto n1 = flhs->GetOp1()->AsNameExpr();
	auto t = flhs->GetType();
	int field = flhs->Field();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_RECORD )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FVi_R, n1, n2, i, field);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FVi_P, n1, n2, i, field);
	else if ( tag == TYPE_OPAQUE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FVi_O, n1, n2, i, field);
	else if ( tag == TYPE_VECTOR )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FVi_V, n1, n2, i, field);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FVi_U, n1, n2, i, field);
	else if ( tag == TYPE_TYPE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FVi_t, n1, n2, i, field);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FVi_D, n1, n2, i, field);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FVi_T, n1, n2, i, field);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FVi_N, n1, n2, i, field);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FVi_A, n1, n2, i, field);
	else if ( tag == TYPE_ANY )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FVi_a, n1, n2, i, field);
	else if ( tag == TYPE_LIST )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FVi_L, n1, n2, i, field);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FVi_F, n1, n2, i, field);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FVi_S, n1, n2, i, field);
	else if ( tag == TYPE_FILE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FVi_f, n1, n2, i, field);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FVi_I, n1, n2, i, field);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Field_LHS_AssignFCi(const FieldLHSAssignExpr* flhs, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto n = flhs->GetOp1()->AsNameExpr();
	auto t = flhs->GetType();
	int field = flhs->Field();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_RECORD )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FCi_R, n, c, i, field);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FCi_P, n, c, i, field);
	else if ( tag == TYPE_OPAQUE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FCi_O, n, c, i, field);
	else if ( tag == TYPE_VECTOR )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FCi_V, n, c, i, field);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FCi_U, n, c, i, field);
	else if ( tag == TYPE_TYPE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FCi_t, n, c, i, field);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FCi_D, n, c, i, field);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FCi_T, n, c, i, field);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FCi_N, n, c, i, field);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FCi_A, n, c, i, field);
	else if ( tag == TYPE_ANY )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FCi_a, n, c, i, field);
	else if ( tag == TYPE_LIST )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FCi_L, n, c, i, field);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FCi_F, n, c, i, field);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FCi_S, n, c, i, field);
	else if ( tag == TYPE_FILE )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FCi_f, n, c, i, field);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_FIELD_LHS_ASSIGN_FCi_I, n, c, i, field);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::CloneVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	z = GenInst(OP_CLONE_VV, n1, n2);
	z.SetType(n1->GetType());
	z.t2 = n2->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::SizeVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_VECTOR )
		z = GenInst(OP_SIZE_VV_V, n1, n2);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_SIZE_VV_T, n1, n2);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_SIZE_VV_S, n1, n2);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_SIZE_VV_N, n1, n2);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_SIZE_VV_A, n1, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_SIZE_VV_D, n1, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_SIZE_VV_U, n1, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_SIZE_VV_I, n1, n2);
	else
		z = GenInst(OP_SIZE_VV, n1, n2);
	z.SetType(n1->GetType());
	z.t2 = n2->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::SizeVV_field(const NameExpr* n1, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_VECTOR )
		z = GenInst(OP_SIZE_VVi_field_V, n1, n2, i);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_SIZE_VVi_field_T, n1, n2, i);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_SIZE_VVi_field_S, n1, n2, i);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_SIZE_VVi_field_N, n1, n2, i);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_SIZE_VVi_field_A, n1, n2, i);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_SIZE_VVi_field_D, n1, n2, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_SIZE_VVi_field_U, n1, n2, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_SIZE_VVi_field_I, n1, n2, i);
	else
		z = GenInst(OP_SIZE_VVi_field, n1, n2, i);
	z.SetType(n1->GetType());
	z.t2 = n2->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NotVC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NOT_VC_I, n, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NotVC_field(const NameExpr* n, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NOT_VCi_field_I, n, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NotVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NOT_VV_I, n1, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NotVV_field(const NameExpr* n1, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NOT_VVi_field_I, n1, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::ComplementVC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_COMPLEMENT_VC_U, n, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::ComplementVC_field(const NameExpr* n, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_COMPLEMENT_VCi_field_U, n, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::ComplementVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_COMPLEMENT_VV_U, n1, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::ComplementVV_field(const NameExpr* n1, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_COMPLEMENT_VVi_field_U, n1, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::PositiveVC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_POSITIVE_VC_D, n, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_POSITIVE_VC_U, n, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_POSITIVE_VC_I, n, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::PositiveVC_field(const NameExpr* n, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_POSITIVE_VCi_field_D, n, c, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_POSITIVE_VCi_field_U, n, c, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_POSITIVE_VCi_field_I, n, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::PositiveVC_vec(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	auto t = c->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_POSITIVE_VC_vec_D, n, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_POSITIVE_VC_vec_U, n, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_POSITIVE_VC_vec_I, n, c);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::PositiveVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_POSITIVE_VV_D, n1, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_POSITIVE_VV_U, n1, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_POSITIVE_VV_I, n1, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::PositiveVV_field(const NameExpr* n1, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_POSITIVE_VVi_field_D, n1, n2, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_POSITIVE_VVi_field_U, n1, n2, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_POSITIVE_VVi_field_I, n1, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::PositiveVV_vec(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_POSITIVE_VV_vec_D, n1, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_POSITIVE_VV_vec_U, n1, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_POSITIVE_VV_vec_I, n1, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NegateVC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_NEGATE_VC_D, n, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_NEGATE_VC_U, n, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NEGATE_VC_I, n, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NegateVC_field(const NameExpr* n, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_NEGATE_VCi_field_D, n, c, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_NEGATE_VCi_field_U, n, c, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NEGATE_VCi_field_I, n, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NegateVC_vec(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	auto t = c->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_NEGATE_VC_vec_D, n, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_NEGATE_VC_vec_U, n, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NEGATE_VC_vec_I, n, c);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NegateVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_NEGATE_VV_D, n1, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_NEGATE_VV_U, n1, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NEGATE_VV_I, n1, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NegateVV_field(const NameExpr* n1, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_NEGATE_VVi_field_D, n1, n2, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_NEGATE_VVi_field_U, n1, n2, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NEGATE_VVi_field_I, n1, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NegateVV_vec(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_NEGATE_VV_vec_D, n1, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_NEGATE_VV_vec_U, n1, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NEGATE_VV_vec_I, n1, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::IncrIV(const NameExpr* n)
	{
	ZInstI z;
	z = GenInst(OP_INCRI_V, n);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::IncrUV(const NameExpr* n)
	{
	ZInstI z;
	z = GenInst(OP_INCRU_V, n);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::DecrIV(const NameExpr* n)
	{
	ZInstI z;
	z = GenInst(OP_DECRI_V, n);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::DecrUV(const NameExpr* n)
	{
	ZInstI z;
	z = GenInst(OP_DECRU_V, n);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AppendToVC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_APPENDTO_VC, n, c);
	z.SetType(c->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AppendToVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	z = GenInst(OP_APPENDTO_VV, n1, n2);
	z.SetType(n2->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AppendToAnyVecVC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_APPENDTOANYVEC_VC, n, c);
	z.SetType(c->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AppendToAnyVecVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	z = GenInst(OP_APPENDTOANYVEC_VV, n1, n2);
	z.SetType(n2->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::ExtendPatternVC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_EXTENDPATTERN_VC, n, c);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::ExtendPatternVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	z = GenInst(OP_EXTENDPATTERN_VV, n1, n2);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AddVecToVecVC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_ADDVECTOVEC_VC, n, c);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AddVecToVecVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	z = GenInst(OP_ADDVECTOVEC_VV, n1, n2);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AddTableToTableVC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_ADDTABLETOTABLE_VC, n, c);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AddTableToTableVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	z = GenInst(OP_ADDTABLETOTABLE_VV, n1, n2);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::RemoveTableFromTableVC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_REMOVETABLEFROMTABLE_VC, n, c);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::RemoveTableFromTableVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	z = GenInst(OP_REMOVETABLEFROMTABLE_VV, n1, n2);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::CastVC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_CAST_VC, n, c);
	z.SetType(n->GetType());
	z.t2 = c->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::CastVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	z = GenInst(OP_CAST_VV, n1, n2);
	z.SetType(n1->GetType());
	z.t2 = n2->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AddVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_ADD_VCV_S, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_ADD_VCV_D, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_ADD_VCV_U, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_ADD_VCV_I, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AddVCV_field(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_ADD_VCVi_field_S, n1, c, n2, i);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_ADD_VCVi_field_D, n1, c, n2, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_ADD_VCVi_field_U, n1, c, n2, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_ADD_VCVi_field_I, n1, c, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AddVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_ADD_VVC_S, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_ADD_VVC_D, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_ADD_VVC_U, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_ADD_VVC_I, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AddVVC_field(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_ADD_VVCi_field_S, n1, n2, c, i);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_ADD_VVCi_field_D, n1, n2, c, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_ADD_VVCi_field_U, n1, n2, c, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_ADD_VVCi_field_I, n1, n2, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AddVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_ADD_VVV_S, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_ADD_VVV_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_ADD_VVV_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_ADD_VVV_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AddVVV_field(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_ADD_VVVi_field_S, n1, n2, n3, i);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_ADD_VVVi_field_D, n1, n2, n3, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_ADD_VVVi_field_U, n1, n2, n3, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_ADD_VVVi_field_I, n1, n2, n3, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AddVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_ADD_VVV_vec_S, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_ADD_VVV_vec_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_ADD_VVV_vec_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_ADD_VVV_vec_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::SubVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_SUB_VCV_T, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_SUB_VCV_D, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_SUB_VCV_U, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_SUB_VCV_I, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::SubVCV_field(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_SUB_VCVi_field_T, n1, c, n2, i);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_SUB_VCVi_field_D, n1, c, n2, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_SUB_VCVi_field_U, n1, c, n2, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_SUB_VCVi_field_I, n1, c, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::SubVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_SUB_VVC_T, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_SUB_VVC_D, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_SUB_VVC_U, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_SUB_VVC_I, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::SubVVC_field(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_SUB_VVCi_field_T, n1, n2, c, i);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_SUB_VVCi_field_D, n1, n2, c, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_SUB_VVCi_field_U, n1, n2, c, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_SUB_VVCi_field_I, n1, n2, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::SubVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_SUB_VVV_T, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_SUB_VVV_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_SUB_VVV_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_SUB_VVV_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::SubVVV_field(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_SUB_VVVi_field_T, n1, n2, n3, i);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_SUB_VVVi_field_D, n1, n2, n3, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_SUB_VVVi_field_U, n1, n2, n3, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_SUB_VVVi_field_I, n1, n2, n3, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::SubVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_SUB_VVV_vec_T, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_SUB_VVV_vec_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_SUB_VVV_vec_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_SUB_VVV_vec_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::TimesVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_TIMES_VCV_D, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_TIMES_VCV_U, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_TIMES_VCV_I, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::TimesVCV_field(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_TIMES_VCVi_field_D, n1, c, n2, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_TIMES_VCVi_field_U, n1, c, n2, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_TIMES_VCVi_field_I, n1, c, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::TimesVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_TIMES_VVC_D, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_TIMES_VVC_U, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_TIMES_VVC_I, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::TimesVVC_field(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_TIMES_VVCi_field_D, n1, n2, c, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_TIMES_VVCi_field_U, n1, n2, c, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_TIMES_VVCi_field_I, n1, n2, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::TimesVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_TIMES_VVV_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_TIMES_VVV_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_TIMES_VVV_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::TimesVVV_field(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_TIMES_VVVi_field_D, n1, n2, n3, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_TIMES_VVVi_field_U, n1, n2, n3, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_TIMES_VVVi_field_I, n1, n2, n3, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::TimesVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_TIMES_VVV_vec_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_TIMES_VVV_vec_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_TIMES_VVV_vec_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::DivideVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_DIVIDE_VCV_AI, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_DIVIDE_VCV_D, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_DIVIDE_VCV_U, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_DIVIDE_VCV_I, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::DivideVCV_field(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_DIVIDE_VCVi_field_AI, n1, c, n2, i);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_DIVIDE_VCVi_field_D, n1, c, n2, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_DIVIDE_VCVi_field_U, n1, c, n2, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_DIVIDE_VCVi_field_I, n1, c, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::DivideVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_DIVIDE_VVC_AI, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_DIVIDE_VVC_D, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_DIVIDE_VVC_U, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_DIVIDE_VVC_I, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::DivideVVC_field(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_DIVIDE_VVCi_field_AI, n1, n2, c, i);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_DIVIDE_VVCi_field_D, n1, n2, c, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_DIVIDE_VVCi_field_U, n1, n2, c, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_DIVIDE_VVCi_field_I, n1, n2, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::DivideVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_DIVIDE_VVV_AI, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_DIVIDE_VVV_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_DIVIDE_VVV_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_DIVIDE_VVV_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::DivideVVV_field(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_DIVIDE_VVVi_field_AI, n1, n2, n3, i);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_DIVIDE_VVVi_field_D, n1, n2, n3, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_DIVIDE_VVVi_field_U, n1, n2, n3, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_DIVIDE_VVVi_field_I, n1, n2, n3, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::DivideVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_DIVIDE_VVV_vec_AI, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_DIVIDE_VVV_vec_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_DIVIDE_VVV_vec_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_DIVIDE_VVV_vec_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::ModVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_MOD_VCV_U, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_MOD_VCV_I, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::ModVCV_field(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_MOD_VCVi_field_U, n1, c, n2, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_MOD_VCVi_field_I, n1, c, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::ModVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_MOD_VVC_U, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_MOD_VVC_I, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::ModVVC_field(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_MOD_VVCi_field_U, n1, n2, c, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_MOD_VVCi_field_I, n1, n2, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::ModVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_MOD_VVV_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_MOD_VVV_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::ModVVV_field(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_MOD_VVVi_field_U, n1, n2, n3, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_MOD_VVVi_field_I, n1, n2, n3, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::ModVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_MOD_VVV_vec_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_MOD_VVV_vec_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::And_AndVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_AND_AND_VCV_I, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::And_AndVCV_field(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_AND_AND_VCVi_field_I, n1, c, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::And_AndVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_AND_AND_VVC_I, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::And_AndVVC_field(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_AND_AND_VVCi_field_I, n1, n2, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::And_AndVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_AND_AND_VVV_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::And_AndVVV_field(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_AND_AND_VVVi_field_I, n1, n2, n3, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::And_AndVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_AND_AND_VVV_vec_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Or_OrVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_OR_OR_VCV_I, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Or_OrVCV_field(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_OR_OR_VCVi_field_I, n1, c, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Or_OrVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_OR_OR_VVC_I, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Or_OrVVC_field(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_OR_OR_VVCi_field_I, n1, n2, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Or_OrVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_OR_OR_VVV_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Or_OrVVV_field(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_OR_OR_VVVi_field_I, n1, n2, n3, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Or_OrVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_OR_OR_VVV_vec_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AndVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_AND_VCV_T, n1, c, n2);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_AND_VCV_P, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_AND_VCV_U, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AndVCV_field(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_AND_VCVi_field_T, n1, c, n2, i);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_AND_VCVi_field_P, n1, c, n2, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_AND_VCVi_field_U, n1, c, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AndVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_AND_VVC_T, n1, n2, c);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_AND_VVC_P, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_AND_VVC_U, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AndVVC_field(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_AND_VVCi_field_T, n1, n2, c, i);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_AND_VVCi_field_P, n1, n2, c, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_AND_VVCi_field_U, n1, n2, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AndVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_AND_VVV_T, n1, n2, n3);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_AND_VVV_P, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_AND_VVV_U, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AndVVV_field(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_AND_VVVi_field_T, n1, n2, n3, i);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_AND_VVVi_field_P, n1, n2, n3, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_AND_VVVi_field_U, n1, n2, n3, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AndVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_AND_VVV_vec_T, n1, n2, n3);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_AND_VVV_vec_P, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_AND_VVV_vec_U, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::OrVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_OR_VCV_T, n1, c, n2);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_OR_VCV_P, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_OR_VCV_U, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::OrVCV_field(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_OR_VCVi_field_T, n1, c, n2, i);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_OR_VCVi_field_P, n1, c, n2, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_OR_VCVi_field_U, n1, c, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::OrVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_OR_VVC_T, n1, n2, c);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_OR_VVC_P, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_OR_VVC_U, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::OrVVC_field(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_OR_VVCi_field_T, n1, n2, c, i);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_OR_VVCi_field_P, n1, n2, c, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_OR_VVCi_field_U, n1, n2, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::OrVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_OR_VVV_T, n1, n2, n3);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_OR_VVV_P, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_OR_VVV_U, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::OrVVV_field(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_OR_VVVi_field_T, n1, n2, n3, i);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_OR_VVVi_field_P, n1, n2, n3, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_OR_VVVi_field_U, n1, n2, n3, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::OrVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_TABLE )
		z = GenInst(OP_OR_VVV_vec_T, n1, n2, n3);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_OR_VVV_vec_P, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_OR_VVV_vec_U, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::XorVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_XOR_VCV_U, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::XorVCV_field(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_XOR_VCVi_field_U, n1, c, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::XorVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_XOR_VVC_U, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::XorVVC_field(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_XOR_VVCi_field_U, n1, n2, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::XorVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_XOR_VVV_U, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::XorVVV_field(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_XOR_VVVi_field_U, n1, n2, n3, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::XorVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_XOR_VVV_vec_U, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LshiftVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LSHIFT_VCV_U, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LSHIFT_VCV_I, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LshiftVCV_field(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LSHIFT_VCVi_field_U, n1, c, n2, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LSHIFT_VCVi_field_I, n1, c, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LshiftVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LSHIFT_VVC_U, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LSHIFT_VVC_I, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LshiftVVC_field(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LSHIFT_VVCi_field_U, n1, n2, c, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LSHIFT_VVCi_field_I, n1, n2, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LshiftVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LSHIFT_VVV_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LSHIFT_VVV_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LshiftVVV_field(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LSHIFT_VVVi_field_U, n1, n2, n3, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LSHIFT_VVVi_field_I, n1, n2, n3, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LshiftVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LSHIFT_VVV_vec_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LSHIFT_VVV_vec_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::RshiftVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_RSHIFT_VCV_U, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_RSHIFT_VCV_I, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::RshiftVCV_field(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_RSHIFT_VCVi_field_U, n1, c, n2, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_RSHIFT_VCVi_field_I, n1, c, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::RshiftVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_RSHIFT_VVC_U, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_RSHIFT_VVC_I, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::RshiftVVC_field(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_RSHIFT_VVCi_field_U, n1, n2, c, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_RSHIFT_VVCi_field_I, n1, n2, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::RshiftVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_RSHIFT_VVV_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_RSHIFT_VVV_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::RshiftVVV_field(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3, int i)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_RSHIFT_VVVi_field_U, n1, n2, n3, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_RSHIFT_VVVi_field_I, n1, n2, n3, i);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::RshiftVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_RSHIFT_VVV_vec_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_RSHIFT_VVV_vec_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LTVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_LT_VCV_A, n1, c, n2);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_LT_VCV_T, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_LT_VCV_S, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_LT_VCV_D, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LT_VCV_U, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LT_VCV_I, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LTVCV_cond(const ConstExpr* c, const NameExpr* n)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_LT_VCV_cond_A, c, n);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_LT_VCV_cond_T, c, n);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_LT_VCV_cond_S, c, n);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_LT_VCV_cond_D, c, n);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LT_VCV_cond_U, c, n);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LT_VCV_cond_I, c, n);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LTVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_LT_VVC_A, n1, n2, c);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_LT_VVC_T, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_LT_VVC_S, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_LT_VVC_D, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LT_VVC_U, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LT_VVC_I, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LTVVC_cond(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_LT_VVC_cond_A, n, c);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_LT_VVC_cond_T, n, c);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_LT_VVC_cond_S, n, c);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_LT_VVC_cond_D, n, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LT_VVC_cond_U, n, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LT_VVC_cond_I, n, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LTVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_LT_VVV_A, n1, n2, n3);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_LT_VVV_T, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_LT_VVV_S, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_LT_VVV_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LT_VVV_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LT_VVV_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LTVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_LT_VVV_vec_A, n1, n2, n3);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_LT_VVV_vec_T, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_LT_VVV_vec_S, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_LT_VVV_vec_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LT_VVV_vec_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LT_VVV_vec_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LTVVV_cond(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n1->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_LT_VVV_cond_A, n1, n2);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_LT_VVV_cond_T, n1, n2);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_LT_VVV_cond_S, n1, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_LT_VVV_cond_D, n1, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LT_VVV_cond_U, n1, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LT_VVV_cond_I, n1, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LEVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_LE_VCV_A, n1, c, n2);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_LE_VCV_T, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_LE_VCV_S, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_LE_VCV_D, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LE_VCV_U, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LE_VCV_I, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LEVCV_cond(const ConstExpr* c, const NameExpr* n)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_LE_VCV_cond_A, c, n);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_LE_VCV_cond_T, c, n);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_LE_VCV_cond_S, c, n);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_LE_VCV_cond_D, c, n);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LE_VCV_cond_U, c, n);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LE_VCV_cond_I, c, n);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LEVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_LE_VVC_A, n1, n2, c);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_LE_VVC_T, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_LE_VVC_S, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_LE_VVC_D, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LE_VVC_U, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LE_VVC_I, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LEVVC_cond(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_LE_VVC_cond_A, n, c);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_LE_VVC_cond_T, n, c);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_LE_VVC_cond_S, n, c);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_LE_VVC_cond_D, n, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LE_VVC_cond_U, n, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LE_VVC_cond_I, n, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LEVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_LE_VVV_A, n1, n2, n3);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_LE_VVV_T, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_LE_VVV_S, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_LE_VVV_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LE_VVV_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LE_VVV_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LEVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_LE_VVV_vec_A, n1, n2, n3);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_LE_VVV_vec_T, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_LE_VVV_vec_S, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_LE_VVV_vec_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LE_VVV_vec_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LE_VVV_vec_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::LEVVV_cond(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n1->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_LE_VVV_cond_A, n1, n2);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_LE_VVV_cond_T, n1, n2);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_LE_VVV_cond_S, n1, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_LE_VVV_cond_D, n1, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_LE_VVV_cond_U, n1, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_LE_VVV_cond_I, n1, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::EQVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_PATTERN )
		z = GenInst(OP_EQ_VCV_PS, n1, c, n2);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_EQ_VCV_F, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_EQ_VCV_N, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_EQ_VCV_A, n1, c, n2);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_EQ_VCV_T, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_EQ_VCV_S, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_EQ_VCV_D, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_EQ_VCV_U, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_EQ_VCV_I, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::EQVCV_cond(const ConstExpr* c, const NameExpr* n)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_PATTERN )
		z = GenInst(OP_EQ_VCV_cond_PS, c, n);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_EQ_VCV_cond_F, c, n);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_EQ_VCV_cond_N, c, n);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_EQ_VCV_cond_A, c, n);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_EQ_VCV_cond_T, c, n);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_EQ_VCV_cond_S, c, n);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_EQ_VCV_cond_D, c, n);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_EQ_VCV_cond_U, c, n);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_EQ_VCV_cond_I, c, n);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::EQVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_PATTERN )
		z = GenInst(OP_EQ_VVC_PS, n1, n2, c);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_EQ_VVC_F, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_EQ_VVC_N, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_EQ_VVC_A, n1, n2, c);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_EQ_VVC_T, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_EQ_VVC_S, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_EQ_VVC_D, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_EQ_VVC_U, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_EQ_VVC_I, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::EQVVC_cond(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_PATTERN )
		z = GenInst(OP_EQ_VVC_cond_PS, n, c);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_EQ_VVC_cond_F, n, c);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_EQ_VVC_cond_N, n, c);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_EQ_VVC_cond_A, n, c);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_EQ_VVC_cond_T, n, c);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_EQ_VVC_cond_S, n, c);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_EQ_VVC_cond_D, n, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_EQ_VVC_cond_U, n, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_EQ_VVC_cond_I, n, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::EQVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_PATTERN )
		z = GenInst(OP_EQ_VVV_PS, n1, n2, n3);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_EQ_VVV_F, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_EQ_VVV_N, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_EQ_VVV_A, n1, n2, n3);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_EQ_VVV_T, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_EQ_VVV_S, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_EQ_VVV_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_EQ_VVV_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_EQ_VVV_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::EQVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_PATTERN )
		z = GenInst(OP_EQ_VVV_vec_PS, n1, n2, n3);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_EQ_VVV_vec_F, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_EQ_VVV_vec_N, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_EQ_VVV_vec_A, n1, n2, n3);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_EQ_VVV_vec_T, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_EQ_VVV_vec_S, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_EQ_VVV_vec_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_EQ_VVV_vec_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_EQ_VVV_vec_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::EQVVV_cond(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n1->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_PATTERN )
		z = GenInst(OP_EQ_VVV_cond_PS, n1, n2);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_EQ_VVV_cond_F, n1, n2);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_EQ_VVV_cond_N, n1, n2);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_EQ_VVV_cond_A, n1, n2);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_EQ_VVV_cond_T, n1, n2);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_EQ_VVV_cond_S, n1, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_EQ_VVV_cond_D, n1, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_EQ_VVV_cond_U, n1, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_EQ_VVV_cond_I, n1, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NEVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_PATTERN )
		z = GenInst(OP_NE_VCV_PS, n1, c, n2);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_NE_VCV_F, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_NE_VCV_N, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_NE_VCV_A, n1, c, n2);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_NE_VCV_T, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_NE_VCV_S, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_NE_VCV_D, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_NE_VCV_U, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NE_VCV_I, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NEVCV_cond(const ConstExpr* c, const NameExpr* n)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_PATTERN )
		z = GenInst(OP_NE_VCV_cond_PS, c, n);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_NE_VCV_cond_F, c, n);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_NE_VCV_cond_N, c, n);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_NE_VCV_cond_A, c, n);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_NE_VCV_cond_T, c, n);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_NE_VCV_cond_S, c, n);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_NE_VCV_cond_D, c, n);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_NE_VCV_cond_U, c, n);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NE_VCV_cond_I, c, n);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NEVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_PATTERN )
		z = GenInst(OP_NE_VVC_PS, n1, n2, c);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_NE_VVC_F, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_NE_VVC_N, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_NE_VVC_A, n1, n2, c);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_NE_VVC_T, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_NE_VVC_S, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_NE_VVC_D, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_NE_VVC_U, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NE_VVC_I, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NEVVC_cond(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_PATTERN )
		z = GenInst(OP_NE_VVC_cond_PS, n, c);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_NE_VVC_cond_F, n, c);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_NE_VVC_cond_N, n, c);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_NE_VVC_cond_A, n, c);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_NE_VVC_cond_T, n, c);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_NE_VVC_cond_S, n, c);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_NE_VVC_cond_D, n, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_NE_VVC_cond_U, n, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NE_VVC_cond_I, n, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NEVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_PATTERN )
		z = GenInst(OP_NE_VVV_PS, n1, n2, n3);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_NE_VVV_F, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_NE_VVV_N, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_NE_VVV_A, n1, n2, n3);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_NE_VVV_T, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_NE_VVV_S, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_NE_VVV_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_NE_VVV_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NE_VVV_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NEVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_PATTERN )
		z = GenInst(OP_NE_VVV_vec_PS, n1, n2, n3);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_NE_VVV_vec_F, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_NE_VVV_vec_N, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_NE_VVV_vec_A, n1, n2, n3);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_NE_VVV_vec_T, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_NE_VVV_vec_S, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_NE_VVV_vec_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_NE_VVV_vec_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NE_VVV_vec_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::NEVVV_cond(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n1->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_PATTERN )
		z = GenInst(OP_NE_VVV_cond_PS, n1, n2);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_NE_VVV_cond_F, n1, n2);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_NE_VVV_cond_N, n1, n2);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_NE_VVV_cond_A, n1, n2);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_NE_VVV_cond_T, n1, n2);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_NE_VVV_cond_S, n1, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_NE_VVV_cond_D, n1, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_NE_VVV_cond_U, n1, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_NE_VVV_cond_I, n1, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::GEVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_GE_VCV_A, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_GE_VCV_S, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_GE_VCV_D, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_GE_VCV_U, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_GE_VCV_I, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::GEVCV_cond(const ConstExpr* c, const NameExpr* n)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_GE_VCV_cond_A, c, n);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_GE_VCV_cond_S, c, n);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_GE_VCV_cond_D, c, n);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_GE_VCV_cond_U, c, n);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_GE_VCV_cond_I, c, n);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::GEVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_GE_VVC_A, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_GE_VVC_S, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_GE_VVC_D, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_GE_VVC_U, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_GE_VVC_I, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::GEVVC_cond(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_GE_VVC_cond_A, n, c);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_GE_VVC_cond_S, n, c);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_GE_VVC_cond_D, n, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_GE_VVC_cond_U, n, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_GE_VVC_cond_I, n, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::GEVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_GE_VVV_A, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_GE_VVV_S, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_GE_VVV_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_GE_VVV_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_GE_VVV_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::GEVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_GE_VVV_vec_A, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_GE_VVV_vec_S, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_GE_VVV_vec_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_GE_VVV_vec_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_GE_VVV_vec_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::GEVVV_cond(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n1->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_GE_VVV_cond_A, n1, n2);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_GE_VVV_cond_S, n1, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_GE_VVV_cond_D, n1, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_GE_VVV_cond_U, n1, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_GE_VVV_cond_I, n1, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::GTVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_GT_VCV_A, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_GT_VCV_S, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_GT_VCV_D, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_GT_VCV_U, n1, c, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_GT_VCV_I, n1, c, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::GTVCV_cond(const ConstExpr* c, const NameExpr* n)
	{
	ZInstI z;
	auto t = c->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_GT_VCV_cond_A, c, n);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_GT_VCV_cond_S, c, n);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_GT_VCV_cond_D, c, n);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_GT_VCV_cond_U, c, n);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_GT_VCV_cond_I, c, n);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::GTVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_GT_VVC_A, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_GT_VVC_S, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_GT_VVC_D, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_GT_VVC_U, n1, n2, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_GT_VVC_I, n1, n2, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::GTVVC_cond(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	auto t = n->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_GT_VVC_cond_A, n, c);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_GT_VVC_cond_S, n, c);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_GT_VVC_cond_D, n, c);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_GT_VVC_cond_U, n, c);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_GT_VVC_cond_I, n, c);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::GTVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_GT_VVV_A, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_GT_VVV_S, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_GT_VVV_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_GT_VVV_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_GT_VVV_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::GTVVV_vec(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	auto t = n2->GetType()->Yield();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_GT_VVV_vec_A, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_GT_VVV_vec_S, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_GT_VVV_vec_D, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_GT_VVV_vec_U, n1, n2, n3);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_GT_VVV_vec_I, n1, n2, n3);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = n1->GetType();
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::GTVVV_cond(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	auto t = n1->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_GT_VVV_cond_A, n1, n2);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_GT_VVV_cond_S, n1, n2);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_GT_VVV_cond_D, n1, n2);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_GT_VVV_cond_U, n1, n2);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_GT_VVV_cond_I, n1, n2);
	else
		reporter->InternalError("bad tag when generating method core");
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::FieldRCi(const NameExpr* n, const ConstExpr* c, int i)
	{
	ZInstI z;
	auto t = n->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_RECORD )
		z = GenInst(OP_FIELD_RCi_R, n, c, i);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_FIELD_RCi_P, n, c, i);
	else if ( tag == TYPE_OPAQUE )
		z = GenInst(OP_FIELD_RCi_O, n, c, i);
	else if ( tag == TYPE_VECTOR )
		z = GenInst(OP_FIELD_RCi_V, n, c, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_FIELD_RCi_U, n, c, i);
	else if ( tag == TYPE_TYPE )
		z = GenInst(OP_FIELD_RCi_t, n, c, i);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_FIELD_RCi_D, n, c, i);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_FIELD_RCi_T, n, c, i);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_FIELD_RCi_N, n, c, i);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_FIELD_RCi_A, n, c, i);
	else if ( tag == TYPE_ANY )
		z = GenInst(OP_FIELD_RCi_a, n, c, i);
	else if ( tag == TYPE_LIST )
		z = GenInst(OP_FIELD_RCi_L, n, c, i);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_FIELD_RCi_F, n, c, i);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_FIELD_RCi_S, n, c, i);
	else if ( tag == TYPE_FILE )
		z = GenInst(OP_FIELD_RCi_f, n, c, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_FIELD_RCi_I, n, c, i);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = t;
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::FieldRVi(const NameExpr* n1, const NameExpr* n2, int i)
	{
	ZInstI z;
	auto t = n1->GetType();
	auto tag = t->Tag();
	auto i_t = t->InternalType();
	if ( tag == TYPE_RECORD )
		z = GenInst(OP_FIELD_RVi_R, n1, n2, i);
	else if ( tag == TYPE_PATTERN )
		z = GenInst(OP_FIELD_RVi_P, n1, n2, i);
	else if ( tag == TYPE_OPAQUE )
		z = GenInst(OP_FIELD_RVi_O, n1, n2, i);
	else if ( tag == TYPE_VECTOR )
		z = GenInst(OP_FIELD_RVi_V, n1, n2, i);
	else if ( i_t == TYPE_INTERNAL_UNSIGNED )
		z = GenInst(OP_FIELD_RVi_U, n1, n2, i);
	else if ( tag == TYPE_TYPE )
		z = GenInst(OP_FIELD_RVi_t, n1, n2, i);
	else if ( i_t == TYPE_INTERNAL_DOUBLE )
		z = GenInst(OP_FIELD_RVi_D, n1, n2, i);
	else if ( tag == TYPE_TABLE )
		z = GenInst(OP_FIELD_RVi_T, n1, n2, i);
	else if ( i_t == TYPE_INTERNAL_SUBNET )
		z = GenInst(OP_FIELD_RVi_N, n1, n2, i);
	else if ( i_t == TYPE_INTERNAL_ADDR )
		z = GenInst(OP_FIELD_RVi_A, n1, n2, i);
	else if ( tag == TYPE_ANY )
		z = GenInst(OP_FIELD_RVi_a, n1, n2, i);
	else if ( tag == TYPE_LIST )
		z = GenInst(OP_FIELD_RVi_L, n1, n2, i);
	else if ( tag == TYPE_FUNC )
		z = GenInst(OP_FIELD_RVi_F, n1, n2, i);
	else if ( i_t == TYPE_INTERNAL_STRING )
		z = GenInst(OP_FIELD_RVi_S, n1, n2, i);
	else if ( tag == TYPE_FILE )
		z = GenInst(OP_FIELD_RVi_f, n1, n2, i);
	else if ( i_t == TYPE_INTERNAL_INT )
		z = GenInst(OP_FIELD_RVi_I, n1, n2, i);
	else
		reporter->InternalError("bad tag when generating method core");
	z.t = t;
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Has_FieldVRi(const NameExpr* n1, const NameExpr* n2, int i)
	{
	ZInstI z;
	z = GenInst(OP_HAS_FIELD_VRi, n1, n2, i);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Has_FieldVRi_field(const NameExpr* n1, const NameExpr* n2, int i1, int i2)
	{
	ZInstI z;
	z = GenInst(OP_HAS_FIELD_VRii_field, n1, n2, i1, i2);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::InVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	return CompileInExpr(n1, n2, n3);
	}

const ZAMStmt ZAMCompiler::InVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	return CompileInExpr(n1, c, n2);
	}

const ZAMStmt ZAMCompiler::InVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	return CompileInExpr(n1, n2, c);
	}

const ZAMStmt ZAMCompiler::L_In_TVLV(const NameExpr* n1, const ListExpr* l, const NameExpr* n2)
	{
	return CompileInExpr(n1, l, n2);
	}

const ZAMStmt ZAMCompiler::L_In_TVLC(const NameExpr* n, const ListExpr* l, const ConstExpr* c)
	{
	return CompileInExpr(n, l, c);
	}

const ZAMStmt ZAMCompiler::L_In_VecVLV(const NameExpr* n1, const ListExpr* l, const NameExpr* n2)
	{
	return CompileInExpr(n1, l, n2);
	}

const ZAMStmt ZAMCompiler::L_In_VecVLC(const NameExpr* n, const ListExpr* l, const ConstExpr* c)
	{
	return CompileInExpr(n, l, c);
	}

const ZAMStmt ZAMCompiler::CondVVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3, const NameExpr* n4)
	{
	ZInstI z;
	z = GenInst(OP_COND_VVVV, n1, n2, n3, n4);
	z.SetType(n3->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::CondVVVC(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_COND_VVVC, n1, n2, n3, c);
	z.SetType(n3->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::CondVVCV(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c, const NameExpr* n3)
	{
	ZInstI z;
	z = GenInst(OP_COND_VVCV, n1, n2, c, n3);
	z.SetType(c->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Bool_Vec_CondVVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3, const NameExpr* n4)
	{
	ZInstI z;
	z = GenInst(OP_BOOL_VEC_COND_VVVV, n1, n2, n3, n4);
	z.SetType(n3->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::CondC1VVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_CONDC1_VVC, n1, n2, c);
	z.SetType(n1->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::CondC2VVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_CONDC2_VVC, n1, n2, c);
	z.SetType(n1->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::IndexVecBoolSelectVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	z = GenInst(OP_INDEXVECBOOLSELECT_VVV, n1, n2, n3);
	z.SetType(n1->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::IndexVecBoolSelectVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	z = GenInst(OP_INDEXVECBOOLSELECT_VCV, n1, c, n2);
	z.SetType(n1->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::IndexVecIntSelectVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	z = GenInst(OP_INDEXVECINTSELECT_VVV, n1, n2, n3);
	z.SetType(n1->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::IndexVecIntSelectVCV(const NameExpr* n1, const ConstExpr* c, const NameExpr* n2)
	{
	ZInstI z;
	z = GenInst(OP_INDEXVECINTSELECT_VCV, n1, c, n2);
	z.SetType(n1->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::IndexVVL(const NameExpr* n1, const NameExpr* n2, const ListExpr* l)
	{
	return CompileIndex(n1, n2, l);
	}

const ZAMStmt ZAMCompiler::IndexVCL(const NameExpr* n, const ConstExpr* c, const ListExpr* l)
	{
	return CompileIndex(n, c, l);
	}

const ZAMStmt ZAMCompiler::AnyIndexVVi(const NameExpr* n1, const NameExpr* n2, int i)
	{
	ZInstI z;
	z = GenInst(OP_ANYINDEX_VVi, n1, n2, i);
	z.SetType(n1->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::To_Any_CoerceVC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_TO_ANY_COERCE_VC, n, c);
	z.SetType(c->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::To_Any_CoerceVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	z = GenInst(OP_TO_ANY_COERCE_VV, n1, n2);
	z.SetType(n2->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::From_Any_CoerceVC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_FROM_ANY_COERCE_VC, n, c);
	z.SetType(n->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::From_Any_CoerceVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	z = GenInst(OP_FROM_ANY_COERCE_VV, n1, n2);
	z.SetType(n1->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::From_Any_Vec_CoerceVC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_FROM_ANY_VEC_COERCE_VC, n, c);
	z.SetType(n->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::From_Any_Vec_CoerceVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	z = GenInst(OP_FROM_ANY_VEC_COERCE_VV, n1, n2);
	z.SetType(n1->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Vector_Elem_AssignVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	z = GenInst(OP_VECTOR_ELEM_ASSIGN_VVV, n1, n2, n3);
	z.SetType(n2->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Any_Vector_Elem_AssignVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	z = GenInst(OP_ANY_VECTOR_ELEM_ASSIGN_VVV, n1, n2, n3);
	z.SetType(n2->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Vector_Elem_Assign_AnyVVV(const NameExpr* n1, const NameExpr* n2, const NameExpr* n3)
	{
	ZInstI z;
	z = GenInst(OP_VECTOR_ELEM_ASSIGN_ANY_VVV, n1, n2, n3);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Vector_Elem_AssignVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_VECTOR_ELEM_ASSIGN_VVC, n1, n2, c);
	z.SetType(c->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Any_Vector_Elem_AssignVVC(const NameExpr* n1, const NameExpr* n2, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_ANY_VECTOR_ELEM_ASSIGN_VVC, n1, n2, c);
	z.SetType(n2->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Vector_Elem_AssignVVi(const NameExpr* n1, const NameExpr* n2, int i)
	{
	ZInstI z;
	z = GenInst(OP_VECTOR_ELEM_ASSIGN_VVi, n1, n2, i);
	z.SetType(n2->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Any_Vector_Elem_AssignVVi(const NameExpr* n1, const NameExpr* n2, int i)
	{
	ZInstI z;
	z = GenInst(OP_ANY_VECTOR_ELEM_ASSIGN_VVi, n1, n2, i);
	z.SetType(n2->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Vector_Elem_Assign_AnyVVi(const NameExpr* n1, const NameExpr* n2, int i)
	{
	ZInstI z;
	z = GenInst(OP_VECTOR_ELEM_ASSIGN_ANY_VVi, n1, n2, i);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::ScheduleViHL(const NameExpr* n, int i, EventHandler* h, const ListExpr* l)
	{
	return CompileSchedule(n, nullptr, i, h, l);
	}

const ZAMStmt ZAMCompiler::ScheduleCiHL(const ConstExpr* c, int i, EventHandler* h, const ListExpr* l)
	{
	return CompileSchedule(nullptr, c, i, h, l);
	}

const ZAMStmt ZAMCompiler::EventHL(EventHandler* h, const ListExpr* l)
	{
	return CompileEvent(h, l);
	}

const ZAMStmt ZAMCompiler::ReturnX()
	{
	ZInstI z;
	z = GenInst(OP_RETURN_X);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::ReturnV(const NameExpr* n)
	{
	ZInstI z;
	z = GenInst(OP_RETURN_V, n);
	z.SetType(n->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::ReturnC(const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_RETURN_C, c);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::CheckAnyLenVi(const NameExpr* n, int i)
	{
	ZInstI z;
	z = GenInst(OP_CHECKANYLEN_Vi, n, i);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::PrintO(OpaqueVals* v)
	{
	ZInstI z;
	z = ZInstI(OP_PRINT_O);
	z.aux = v->aux;
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Print1V(const NameExpr* n)
	{
	ZInstI z;
	z = GenInst(OP_PRINT1_V, n);
	z.SetType(n->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Print1C(const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_PRINT1_C, c);
	z.SetType(c->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AddStmtVO(const NameExpr* n, OpaqueVals* v)
	{
	ZInstI z;
	z = ZInstI(OP_ADDSTMT_VO, Frame1Slot(n, OP_ADDSTMT_VO));
	z.aux = v->aux;
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AddStmt1VV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	z = GenInst(OP_ADDSTMT1_VV, n1, n2);
	z.SetType(n2->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::AddStmt1VC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_ADDSTMT1_VC, n, c);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::DelTableVO(const NameExpr* n, OpaqueVals* v)
	{
	ZInstI z;
	z = ZInstI(OP_DELTABLE_VO, Frame1Slot(n, OP_DELTABLE_VO));
	z.aux = v->aux;
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::DelFieldVi(const NameExpr* n, int i)
	{
	ZInstI z;
	z = GenInst(OP_DELFIELD_Vi, n, i);
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Assign_AnyVV(const NameExpr* n1, const NameExpr* n2)
	{
	ZInstI z;
	z = GenInst(OP_ASSIGN_ANY_VV, n1, n2);
	z.SetType(n2->GetType());
	return AddInst(z);
	}

const ZAMStmt ZAMCompiler::Assign_AnyVC(const NameExpr* n, const ConstExpr* c)
	{
	ZInstI z;
	z = GenInst(OP_ASSIGN_ANY_VC, n, c);
	z.SetType(c->GetType());
	return AddInst(z);
	}

